﻿/*
* Debugging lib based on liquid crystal kner 2015
* 
* uses http://www.dfrobot.com/wiki/index.php/Arduino_LCD_KeyPad_Shield_%28SKU:_DFR0009%29
* for debugging 
*/


#ifndef _DEBUG_H
#define _DEBUG_H

#ifdef DEBUG_LCD
// this section is not compiled in debug mode

#include <Arduino.h>
#include <LiquidCrystal_I2C.h>

#define NCHARS 16
#define LINE1 0
#define LINE2 1

#define L(x)	debug.setCursor(0,0);debug.print(x);
#define L2(x,y) debug.setCursor(0,0);debug.print(x,y);
#define R(x)	debug.setCursor(12,0);debug.print(x);
#define R2(x,y) debug.rightToLeft();debug.setCursor(12,0);debug.print(x,y);
#define M(x)	debug.setCursor(8,0);debug.print(x);
#define M2(x,y) debug.setCursor(8,0);debug.print(x,y);
#define T(x)	debug.trace(x);
#define THEX(x)	debug.tracehex(x);
#define TDEC(x)	debug.tracedec(x);
#define WAIT_MS(x)	debug.wait_ms(x);


class Debug: public LiquidCrystal_I2C{
	private:
	
	char buffer[NCHARS];
	
	public: 
	Debug(): LiquidCrystal_I2C(0x3F,16,2){
		
	}
	
	/*
	trace adds new strings at the beginning and rotates the string to the right
	*/
	void trace(const char* s){
		uint8_t len=strlen(s);
		if (len>=NCHARS) len=NCHARS-1; 
		//rotate right by len positions
		for (uint8_t pos=NCHARS-1;pos>0; pos--)  buffer[pos]=buffer[pos-len-1];
		//copy s to start of buffer
		for (uint8_t i=0; i<len;i++) buffer[i]=s[i];
		//separator
		buffer[len]='|';
		LiquidCrystal_I2C::setCursor(0,1);
		LiquidCrystal_I2C::print(buffer);		
	}
	void tracehex(uint16_t u){
		char buf[6];
		itoa(u,buf,16); 
		trace(buf); 
	}
	void tracedec(uint16_t u){
		char buf[8];
		itoa(u,buf,10);
		trace(buf);
	}
		
	void wait_ms(uint16_t time){
		for (volatile long int i=0; i<(time*150L); i++); //just wait
	}
	
};

Debug debug;
#else
// no flash memory used if undef DEBUG_LCD

	#define L(x)         ((void) 0);
	#define L2(x,y)      ((void) 0);
	#define R(x)         ((void) 0);
	#define R2(x,y)      ((void) 0);
	#define M(x)         ((void) 0);
	#define M2(x,y)      ((void) 0);
	#define M2(x,y)      ((void) 0);
	#define T(x)     ((void) 0);
	#define THEX(x)     ((void) 0);
	#define TDEC(x)     ((void) 0);
	#define WAIT_MS(x)   ((void) 0);
	
#endif /* DEBUG_LCD */
#endif /* _DEBUG_H */